using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using Microsoft.Win32;
using System.IO;

///
/// BAMGUI. Made by Rikard Alard
/// 
/// GUI for working with bm.exe
///
namespace BAMGUI
{
    public partial class Form1 : Form
    {
        #region Private Members, Constants, Enums, Types etc

        private string m_trackingPath = string.Empty;
        private const bool KeepOldLogText = true;
        private const bool OverwriteOldLogText = false;
        private string m_dataBase = string.Empty;
        private string m_server = string.Empty;
        private TreeNode m_selectedNode = null; // Selected right click in treenode (I know this is ugly, but could not figure out which clicked in event for context menu)

        private enum ItemType
        {
            Activity,
            Views,
            Alerts,
            Accounts,
            Config
        }

        private enum NodeType
        {
            View,
            Alert,
            Account
        }

        #endregion

        #region Constructor

        public Form1()
        {
            InitializeComponent();

            m_dataBase = BAMGUI.Default.Db;
            if (BAMGUI.Default.Server != string.Empty)
            {
                m_server = BAMGUI.Default.Server;
            }
            else
            {
               m_server = System.Environment.MachineName;
            }
            m_databaseLbl.Text = m_dataBase;
            m_serverLbl.Text = m_server;
            m_bamDefFileTextBox.Text = BAMGUI.Default.InitialBAMDefinition;
            m_trackingPath = BAMGUI.Default.BMPath;
            m_BAMConfigFileTextBox.Text = BAMGUI.Default.InitialFolder + "MyConfig.xml";
            FillItemsListTreeView(OverwriteOldLogText);

            m_versionComboBox.SelectedIndex = 0;
            m_versionComboBox.Enabled = false;
        }

        #endregion

        #region Private methods

        private string ExecuteCommand(string cmd, object sender, bool keepOldLogText)
        {
            String finalOut = string.Empty;

            try
            {
                this.Cursor = Cursors.WaitCursor;

                if (sender != null && sender is Button)
                {
                    Button btn = (Button)sender;
                    btn.Enabled = false;
                }

                // Check if bm.exe exists
                System.IO.FileInfo fi = new System.IO.FileInfo(m_trackingPath);
                if (!fi.Exists)
                {
                    throw new Exception(string.Format("Could not find bm.exe at path '{0}'. Please change BMPath in config file!", m_trackingPath));
                }

                // Try to get encoding 
                int oemCodePage = 0;
                try
                {
                    RegistryKey codepageKey = Registry.LocalMachine.OpenSubKey(@"SYSTEM\CurrentControlSet\Control\Nls\CodePage");
                    oemCodePage = int.Parse((string)codepageKey.GetValue("OEMCP"));
                }
                catch
                {}

                DoLog(string.Format("Executing command: {0} {1}", m_trackingPath, cmd), keepOldLogText);
                string bamDefFile = m_bamDefFileTextBox.Text.Trim();
                Process bmProcess = new Process();
                ProcessStartInfo bsi = new ProcessStartInfo(m_trackingPath, cmd);
                bsi.RedirectStandardOutput = true;
                bsi.UseShellExecute = false;
                bsi.CreateNoWindow = true;
                bmProcess.StartInfo = bsi;
                bmProcess.Start();

                if (oemCodePage == 0)
                {
                    finalOut = bmProcess.StandardOutput.ReadToEnd();
                }
                else
                {
                    StreamReader newStandardOutput = new StreamReader(bmProcess.StandardOutput.BaseStream, Encoding.GetEncoding(oemCodePage));
                    finalOut = newStandardOutput.ReadToEnd();
                }

                bmProcess.WaitForExit();
                if (bmProcess.HasExited)
                {
                    DoLog(finalOut, true);
                }
                else
                {
                    throw new Exception("Error");
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                if (sender != null && sender is Button)
                {
                    Button btn = (Button)sender;
                    btn.Enabled = true;
                }

                this.Cursor = Cursors.Default;
            }

            if (finalOut.Contains("ERROR: "))
            {
                MessageBox.Show(finalOut.Substring(finalOut.IndexOf("ERROR: ")), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            return finalOut;
        }

        private void SelectItemType(ItemType itemType)
        {
            m_itemsListTreeView.Tag = itemType;

            switch (itemType)
            {
                case ItemType.Activity:
                    m_listGroupBox.Text = "Activities";
                    break;
                case ItemType.Views:
                    m_listGroupBox.Text = "Views";
                    break;
                case ItemType.Alerts:
                    m_listGroupBox.Text = "Alerts";
                    break;
                case ItemType.Accounts:
                    m_listGroupBox.Text = "Accounts";
                    break;
                case ItemType.Config:
                    m_listGroupBox.Text = "Config";
                    break;
            }
        }

        private void FillItemsListTreeView(bool KeepTheOldLogText)
        {
            m_itemsListTreeView.Nodes.Clear();

            string tmp;
            int index;
            string[] items;

            switch (m_itemsTabControl.SelectedTab.Name)
            {
               case "m_activitiesTabPage":
                    m_activityNameTextBox.Text = string.Empty;
                    SelectItemType(ItemType.Activity);
                    tmp = ExecuteCommand(string.Format("get-activities -Server:\"{0}\" -Database:\"{1}\"", m_server, m_dataBase), null, KeepTheOldLogText);
                    index = tmp.IndexOf("All activities in the database:\r\n");
                    if (index > 0)
                    {
                        tmp = tmp.Substring(index + 33);
                        items = tmp.Split('\r');
                        foreach (string activity in items)
                        {
                            if (activity != "\n" && activity != "\r" && activity != "\r\n")
                                m_itemsListTreeView.Nodes.Add(activity.Replace("\n", string.Empty));
                        }
                    }
                    else
                    {
                        m_itemsListTreeView.Nodes.Add("No Activities are defined");
                    }
                    break;

                case "m_viewsTabPage":
                    m_viewNameTextBox.Text = string.Empty;
                    SelectItemType(ItemType.Views);
                    tmp = ExecuteCommand(string.Format("get-views -Server:\"{0}\" -Database:\"{1}\"", m_server, m_dataBase), null, KeepTheOldLogText);
                    index = tmp.IndexOf("All views in the database:\r\n");
                    if (index > 0)
                    {
                        tmp = tmp.Substring(index + 28);
                        items = tmp.Split('\r');
                        foreach (string view in items)
                        {
                            if (view != "\n" && view != "\r" && view != "\r\n")
                                m_itemsListTreeView.Nodes.Add(view.Replace("\n", string.Empty));
                        }
                     }
                     else
                     {
                         m_itemsListTreeView.Nodes.Add("No views are defined");
                     }
                    break;

                case "m_alertsTabPage":
                    m_alertNameTextBox.Text = string.Empty;
                    SelectItemType(ItemType.Alerts);
                    tmp = ExecuteCommand(string.Format("get-alerts -Server:\"{0}\" -Database:\"{1}\"", m_server, m_dataBase), null, KeepTheOldLogText);
                    index = tmp.IndexOf("Alerts for view ");
                    if (index > 0)
                    {
                        tmp = tmp.Substring(index);
                        items = tmp.Split('\r');
                        string lastView = string.Empty;
                        TreeNode activeNode = null;
                        foreach (string item in items)
                        {
                            if (item.Equals("\n") || item.Equals("\r") || item.Equals("\r\n"))
                                continue;
                            if (item.IndexOf("Alerts for view ") > -1)
                            {
                                // Parse new view name
                                string viewName = item.Substring(item.IndexOf('\'') + 1);
                                viewName = viewName.TrimEnd(new char[] { '\'', ':' });
                                TreeNode newViewNode = new TreeNode(viewName);
                                activeNode = newViewNode;
                                newViewNode.Tag = NodeType.View;
                                m_itemsListTreeView.Nodes.Add(newViewNode);
                            }
                            else
                            {
                                activeNode.Nodes.Add(item.TrimStart('\n'));
                            }
                        }
                        m_itemsListTreeView.ExpandAll();
                    }
                    else
                    {
                        m_itemsListTreeView.Nodes.Add("No alerts are defined");
                    }
                    break;

                case "m_accountsTabPage":
                    m_accountsViewNameTextBox.Text = string.Empty;
                    m_accountsAccountNameTextBox.Text = string.Empty;
                    SelectItemType(ItemType.Accounts);
                    tmp = ExecuteCommand(string.Format("get-views -Server:\"{0}\" -Database:\"{1}\"", m_server, m_dataBase), null, KeepTheOldLogText);
                    index = tmp.IndexOf("All views in the database:\r\n");
                    if (index > 0)
                    {
                        tmp = tmp.Substring(index + 28);
                        items = tmp.Split('\r');
                        foreach (string view in items)
                        {
                            if (view.Equals("\n") || view.Equals("\r") || view.Equals("\r\n"))
                                continue;

                            string viewName = view.Replace("\n", string.Empty);
                            TreeNode newNode = new TreeNode(viewName);
                            m_itemsListTreeView.Nodes.Add(newNode);
                            newNode.Tag = NodeType.Account;
                            tmp = ExecuteCommand(string.Format("get-accounts -View:\"{0}\" -Server:\"{1}\" -Database:\"{2}\"",viewName, m_server, m_dataBase), null, KeepOldLogText);
                            string accountsStr = string.Format("Accounts for view '{0}':\r\n", viewName);
                            index = tmp.IndexOf(accountsStr);
                            if (index > 0)
                            {
                                tmp = tmp.Substring(index + accountsStr.Length);
                                items = tmp.Split('\r');
                                foreach (string account in items)
                                {
                                    if (account.Equals("\n") || account.Equals("\r") || account.Equals("\r\n"))
                                        continue;

                                    newNode.Nodes.Add(account.Replace("\n", string.Empty));
                                }
                            }
                            else
                            {
                                newNode.Nodes.Add("No accounts are defined for this view");
                            }
                        }
                    }
                    else
                    {
                        m_itemsListTreeView.Nodes.Add("No views are defined");
                    }
                    break;

                case "m_configTabPage":
                    m_BAMConfigFileTextBox.Text = string.Empty;
                    SelectItemType(ItemType.Config);
                    break;
                
            }

            m_itemsListTreeView.ExpandAll();
        }

        private void DoLog(string s1)
        {
            m_logTextBox.Text = s1;
            Application.DoEvents();
        }

        private void DoLog(string s1, bool keepOldLogText)
        {
            if (keepOldLogText)
                m_logTextBox.Text += Environment.NewLine + s1;
            else
                m_logTextBox.Text = s1;
        }


        #endregion

        #region Events

        private void m_deployBamDefBtn_Click(object sender, EventArgs e)
        {
            //if (MessageBox.Show(string.Format("Are you sure you want to deploy bam definition '{0}'?", m_bamDefFileTextBox.Text.Trim()), "Deploy bam definition?", MessageBoxButtons.OKCancel) == DialogResult.Cancel)
            //   return;

            string cmd = "deploy-all -DefinitionFile:\"{0}\" -Server:\"{1}\" -Database:\"{2}\"";
            cmd = string.Format(cmd, m_bamDefFileTextBox.Text.Trim(), m_server, m_dataBase);
            ExecuteCommand(cmd, sender, OverwriteOldLogText);
            FillItemsListTreeView(KeepOldLogText);
        }

        private void m_updateBamDefBtn_Click(object sender, EventArgs e)
        {
            //if (MessageBox.Show(string.Format("Are you sure you want to update bam definition '{0}'?", m_bamDefFileTextBox.Text.Trim()), "Update bam definition?", MessageBoxButtons.OKCancel) == DialogResult.Cancel)
            //    return;

            string cmd = "update-all -DefinitionFile:\"{0}\" -Server:\"{1}\" -Database:\"{2}\"";
            cmd = string.Format(cmd, m_bamDefFileTextBox.Text.Trim(), m_server, m_dataBase);
            ExecuteCommand(cmd, sender, OverwriteOldLogText);
            FillItemsListTreeView(KeepOldLogText);
        }

        private void m_removeBamDefBtn_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(string.Format("Are you sure you want to delete bam definition '{0}'?", m_bamDefFileTextBox.Text.Trim()), "Delete bam definition?", MessageBoxButtons.OKCancel) == DialogResult.Cancel)
                return;

            string cmd = "remove-all -DefinitionFile:\"{0}\" -Server:\"{1}\" -Database:\"{2}\"";
            cmd = string.Format(cmd, m_bamDefFileTextBox.Text.Trim(), m_server, m_dataBase);
            ExecuteCommand(cmd, sender, OverwriteOldLogText);
            FillItemsListTreeView(KeepOldLogText);
        }

        private void m_browseBAMDefBtn_Click(object sender, EventArgs e)
        {
            OpenFileDialog openBrowseFileDialog = new OpenFileDialog();
            openBrowseFileDialog.InitialDirectory = BAMGUI.Default.InitialFolder;
            openBrowseFileDialog.Filter = "BizTalk BAM Definition file (*.xls)|*.xls|All files (*.*)|*.*";
            openBrowseFileDialog.FilterIndex = 1;
            openBrowseFileDialog.RestoreDirectory = true;
            openBrowseFileDialog.Title = "Open BizTalk BAM Definition file";
            if (openBrowseFileDialog.ShowDialog() == DialogResult.OK)
                m_bamDefFileTextBox.Text = openBrowseFileDialog.FileName;

        }

        private void m_executeCommandBtn_Click(object sender, EventArgs e)
        {
            ExecuteCommand(m_commandTextBox.Text.Trim(), sender, OverwriteOldLogText);
            FillItemsListTreeView(KeepOldLogText);
        }

        private void m_helpBtn_Click(object sender, EventArgs e)
        {
            string cmd = "help";
            ExecuteCommand(cmd, sender, OverwriteOldLogText);
        }

        private void m_itemsTabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            FillItemsListTreeView(OverwriteOldLogText);
        }

        private void m_updateItemsListBtn_Click(object sender, EventArgs e)
        {
            FillItemsListTreeView(OverwriteOldLogText);
        }

        private void m_copyToItemTabBtn_Click(object sender, EventArgs e)
        {
            ItemType itemType = (ItemType)m_itemsListTreeView.Tag;
            if (m_itemsListTreeView.SelectedNode != null)
            {
                switch (itemType)
                {
                    case ItemType.Activity:
                        m_itemsTabControl.SelectedTab = m_activitiesTabPage;
                        m_activityNameTextBox.Text = m_itemsListTreeView.SelectedNode.Text;
                        break;
                    case ItemType.Views:
                        m_itemsTabControl.SelectedTab = m_viewsTabPage;
                        m_viewNameTextBox.Text = m_itemsListTreeView.SelectedNode.Text;
                        break;
                    case ItemType.Alerts:
                        m_itemsTabControl.SelectedTab = m_alertsTabPage;
                        if (m_itemsListTreeView.SelectedNode.Tag == null)
                            m_alertNameTextBox.Text = m_itemsListTreeView.SelectedNode.Parent.Text;
                        else
                            m_alertNameTextBox.Text = m_itemsListTreeView.SelectedNode.Text;
                        break;
                    case ItemType.Accounts:
                        m_itemsTabControl.SelectedTab = m_accountsTabPage;
                        if (m_itemsListTreeView.SelectedNode.Tag == null)
                        {
                            m_accountsViewNameTextBox.Text = m_itemsListTreeView.SelectedNode.Parent.Text;
                            m_accountsAccountNameTextBox.Text = m_itemsListTreeView.SelectedNode.Text;
                        }
                        else
                        {
                            m_accountsViewNameTextBox.Text = string.Empty;
                            m_accountsAccountNameTextBox.Text = string.Empty;
                        }
                        break;
                }
            }
        }

        private void m_removeActivityBtn_Click(object sender, EventArgs e)
        {
            RemoveActivity(m_activityNameTextBox.Text.Trim(), sender);
        }

        private void RemoveActivity(string activityName, object sender)
        {
            if (MessageBox.Show(string.Format("Are you sure you want to delete activity '{0}'?", activityName), "Delete activity?", MessageBoxButtons.OKCancel) == DialogResult.Cancel)
                return;
            string cmd = "remove-activity -Name:\"{0}\" -Server:\"{1}\" -Database:\"{2}\"";
            cmd = string.Format(cmd, activityName, m_server, m_dataBase);
            ExecuteCommand(cmd, sender, OverwriteOldLogText);
            FillItemsListTreeView(KeepOldLogText);
        }

        private void m_itemsListTreeView_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                // Show context menu
                ContextMenu menu = new ContextMenu();
                MenuItem item;

                switch ((ItemType)m_itemsListTreeView.Tag)
                {
                    case ItemType.Activity:
                        item = new MenuItem("Remove");
                        menu.MenuItems.Add(item);
                        item.Click += new System.EventHandler(this.RemoveActivityMenuItem_Click);
                        break;

                    case ItemType.Views:
                        item = new MenuItem("Remove");
                        menu.MenuItems.Add(item);
                        item.Click += new System.EventHandler(this.RemoveViewMenuItem_Click);
                        break;

                    case ItemType.Alerts:
                        item = new MenuItem("Remove");
                        menu.MenuItems.Add(item);
                        item.Click += new System.EventHandler(this.RemoveAlertsMenuItem_Click);
                        break;

                    default:
                        break;
                }

                if (menu.MenuItems != null && menu.MenuItems.Count > 0)
                    m_itemsListTreeView.ContextMenu = menu;
                else
                    m_itemsListTreeView.ContextMenu = null;
              }
        }

        private void RemoveActivityMenuItem_Click(object sender, System.EventArgs e)
        {
            RemoveActivity(m_selectedNode.Text, null);
        }

        private void RemoveViewMenuItem_Click(object sender, System.EventArgs e)
        {
            RemoveView(m_selectedNode.Text, null);
        }

        private void RemoveAlertsMenuItem_Click(object sender, System.EventArgs e)
        {
            string selectedNodeName;
            if (m_selectedNode.Tag == null)
                selectedNodeName = m_selectedNode.Parent.Text;
            else
                selectedNodeName = m_selectedNode.Text;

            RemoveAlert(selectedNodeName, null);
                                   
        }
        
        private void m_forcedRemoveActivityBtn_Click(object sender, EventArgs e)
        {
            // Remove associated views/alerts first (more to remove first? accounts etc?)
        }

        private void m_removeViewBtn_Click(object sender, EventArgs e)
        {
            RemoveView(m_viewNameTextBox.Text.Trim(), sender);
        }

        private void RemoveView(string viewName, object sender)
        {
            if (MessageBox.Show(string.Format("Are you sure you want to delete view '{0}'?", viewName), "Delete view?", MessageBoxButtons.OKCancel) == DialogResult.Cancel)
                return;
            string cmd = "remove-view -Name:\"{0}\" -Server:\"{1}\" -Database:\"{2}\"";
            cmd = string.Format(cmd, viewName, m_server, m_dataBase);
            ExecuteCommand(cmd, sender, OverwriteOldLogText);
            FillItemsListTreeView(KeepOldLogText);
        }

        private void m_removeAlertBtn_Click(object sender, EventArgs e)
        {
            RemoveAlert(m_alertNameTextBox.Text.Trim(), sender);
        }

        private void RemoveAlert(string viewName, object sender)
        {
            if (MessageBox.Show(string.Format("Are you sure you want to delete all alerts for view '{0}'?", viewName), "Delete alerts?", MessageBoxButtons.OKCancel) == DialogResult.Cancel)
                return;
            string cmd = "remove-alerts -View:\"{0}\" -Server:\"{1}\" -Database:\"{2}\"";
            cmd = string.Format(cmd, viewName, m_server, m_dataBase);
            ExecuteCommand(cmd, sender, OverwriteOldLogText);
            FillItemsListTreeView(KeepOldLogText);
        }

        private void m_browseBAMConfigFileBtn_Click(object sender, EventArgs e)
        {
            OpenFileDialog openBrowseFileDialog = new OpenFileDialog();
            openBrowseFileDialog.InitialDirectory = BAMGUI.Default.InitialFolder;
            openBrowseFileDialog.Filter = "BizTalk BAM Configuration Xml file (*.xml)|*.xml|All files (*.*)|*.*";
            openBrowseFileDialog.FilterIndex = 1;
            openBrowseFileDialog.RestoreDirectory = true;
            openBrowseFileDialog.Title = "Open BizTalk BAM Configuration Xml file";
            if (openBrowseFileDialog.ShowDialog() == DialogResult.OK)
                m_BAMConfigFileTextBox.Text = openBrowseFileDialog.FileName;
        }

        private void m_exportBAMConfigFileBtn_Click(object sender, EventArgs e)
        {
            string cmd = "get-config -Filename:\"{0}\" -Server:\"{1}\" -Database:\"{2}\"";
            cmd = string.Format(cmd, m_BAMConfigFileTextBox.Text.Trim(), m_server, m_dataBase);
            ExecuteCommand(cmd, sender, OverwriteOldLogText);
        }

        private void m_updateBAMConfigFileBtn_Click(object sender, EventArgs e)
        {
            string cmd = "update-config -Filename:\"{0}\"";
            cmd = string.Format(cmd, m_BAMConfigFileTextBox.Text.Trim());
            ExecuteCommand(cmd, sender, OverwriteOldLogText);
        }

        private void m_addAccountBtn_Click(object sender, EventArgs e)
        {
            string cmd = string.Format("add-account -AccountName:\"{0}\" -View:\"{1}\" -Server:\"{2}\" -Database:\"{3}\"", m_accountsAccountNameTextBox.Text.Trim(), m_accountsViewNameTextBox.Text.Trim(), m_server, m_dataBase);
            cmd = string.Format(cmd, m_bamDefFileTextBox.Text.Trim());
            ExecuteCommand(cmd, sender, OverwriteOldLogText);
            FillItemsListTreeView(KeepOldLogText);
        }

        private void m_removeAccountBtn_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(string.Format("Are you sure you want to remove account '{0}' from view '{1}'?", m_accountsAccountNameTextBox.Text.Trim(), m_accountsViewNameTextBox.Text.Trim()), "Remove account?", MessageBoxButtons.OKCancel) == DialogResult.Cancel)
                return;

            string cmd = string.Format("remove-account -AccountName:\"{0}\" -View:\"{1}\" -Server:\"{2}\" -Database:\"{3}\"", m_accountsAccountNameTextBox.Text.Trim(), m_accountsViewNameTextBox.Text.Trim(), m_server, m_dataBase);
            cmd = string.Format(cmd, m_bamDefFileTextBox.Text.Trim());
            ExecuteCommand(cmd, sender, OverwriteOldLogText);
            FillItemsListTreeView(KeepOldLogText);
        }

        private void m_updateLivedataworkbookBtn_Click(object sender, EventArgs e)
        {
            string cmd = string.Format("update-livedataworkbook -Name:\"{0}\" -Server:\"{1}\" -Database:\"{2}\"", m_bamDefFileTextBox.Text, m_server, m_dataBase);
            ExecuteCommand(cmd, sender, OverwriteOldLogText);
        }

        private void m_regenerateLivedataworkbookBtn_Click(object sender, EventArgs e)
        {
            string cmd = string.Format("regenerate-livedataworkbook -WorkbookName:\"{0}\" -Server:\"{1}\" -Database:\"{2}\"", m_bamDefFileTextBox.Text, m_server, m_dataBase);
            ExecuteCommand(cmd, sender, OverwriteOldLogText);
        }


        private void Form1_OnClosed(object sender, EventArgs e)
        {
            // Save settings to setting file
            /*if (BAMGUI.Default.Server == string.Empty)
                BAMGUI.Default.Server = System.Environment.MachineName;

            BAMGUI.Default.InitialBAMDefinition = m_bamDefFileTextBox.Text;*/
        }


        private void m_itemsListTreeView_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            m_selectedNode = e.Node;
        }


        #endregion

 
       

       

       
    }
}